"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.main = void 0;
const chalk_1 = __importDefault(require("chalk"));
const cli_progress_1 = __importDefault(require("cli-progress"));
const fs_1 = __importDefault(require("fs"));
const optionator_1 = __importDefault(require("optionator"));
const path_1 = __importDefault(require("path"));
const processDatabase_1 = __importDefault(require("../processDatabase"));
// @ts-ignore
// eslint-disable-next-line @typescript-eslint/no-var-requires
const { version } = require('../../package.json');
function main() {
    return __awaiter(this, void 0, void 0, function* () {
        console.info(chalk_1.default.greenBright('Kanel'));
        const o = (0, optionator_1.default)({
            prepend: 'Usage: kanel [options]',
            append: `Version ${version}`,
            options: [
                {
                    option: 'help',
                    alias: 'h',
                    type: 'Boolean',
                    description: 'displays help',
                },
                {
                    option: 'version',
                    alias: 'v',
                    type: 'Boolean',
                    description: 'displays version',
                },
                {
                    option: 'config',
                    alias: 'c',
                    type: 'path::String',
                    description: 'Use this configuration, overriding .kanelrc.js config options if present',
                },
                {
                    option: 'database',
                    alias: 'd',
                    type: 'String',
                    description: 'Database connection string. Will override the connection field in the config file if present',
                },
                {
                    option: 'output',
                    alias: 'o',
                    type: 'path::String',
                    description: 'Output directory. Will override the output field in the config file if present',
                },
            ],
        });
        let options;
        try {
            options = o.parseArgv(process.argv);
        }
        catch (error) {
            console.error(error.message);
            process.exit(1);
        }
        if (options.help) {
            console.info(o.generateHelp());
            process.exit(0);
        }
        if (options.version) {
            console.info(version);
            process.exit(0);
        }
        let config;
        const configFile = path_1.default.join(process.cwd(), options.config || '.kanelrc.js');
        if (fs_1.default.existsSync(configFile) ||
            fs_1.default.existsSync(configFile + '.js') ||
            fs_1.default.existsSync(configFile + '.json')) {
            console.info(`Using config file: ${configFile}`);
            try {
                config = require(configFile);
            }
            catch (error) {
                console.error('Could not open config file:', error);
                process.exit(1);
            }
        }
        else {
            if (options.config) {
                console.error('Could not open ' + options.config);
                process.exit(1);
            }
            config = { connection: undefined };
        }
        if (options.database) {
            config.connection = options.database;
        }
        if (!config.connection) {
            console.error('No database specified, in config file or command line');
            process.exit(1);
        }
        if (options.output) {
            config.outputPath = options.output;
        }
        if (!config.outputPath) {
            console.error('No output path specified, in config file or command line');
            process.exit(1);
        }
        const bar = new cli_progress_1.default.SingleBar({}, cli_progress_1.default.Presets.shades_classic);
        const progress = {
            onProgressStart: (total) => bar.start(total, 0),
            onProgress: () => bar.increment(),
            onProgressEnd: () => bar.stop(),
        };
        try {
            yield (0, processDatabase_1.default)(config, progress);
            process.exit(0);
        }
        catch (error) {
            console.error(error);
            process.exit(1);
        }
    });
}
exports.main = main;
