"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const tagged_comment_parser_1 = require("tagged-comment-parser");
const escapeName_1 = __importDefault(require("../escapeName"));
const makeMapper = (style, config) => (enumDetails) => {
    // If an enum has a @type tag in the comment,
    // we will use that type instead of a generated one.
    const { tags } = (0, tagged_comment_parser_1.tryParse)(enumDetails.comment);
    if (tags === null || tags === void 0 ? void 0 : tags.type) {
        return undefined;
    }
    const { name, comment, path } = config.getMetadata(enumDetails, undefined, config);
    if (style === 'type') {
        const declaration = {
            declarationType: 'typeDeclaration',
            name,
            comment,
            exportAs: 'default',
            typeDefinition: [
                '',
                ...enumDetails.values.map((value) => `| '${value}'`),
            ],
        };
        return { path, declaration };
    }
    else {
        const declaration = {
            declarationType: 'generic',
            comment,
            lines: [
                `enum ${name} {`,
                ...enumDetails.values.map((value) => `  ${(0, escapeName_1.default)(value)} = '${value}',`),
                '};',
                '',
                `export default ${name};`,
            ],
        };
        return { path, declaration };
    }
};
const makeEnumsGenerator = (config) => (schema, outputAcc) => {
    var _a, _b;
    const declarations = (_b = (_a = schema.enums) === null || _a === void 0 ? void 0 : _a.map(makeMapper(config.enumStyle, config))) !== null && _b !== void 0 ? _b : [];
    return declarations.reduce((acc, elem) => {
        if (elem === undefined)
            return acc;
        const { path, declaration } = elem;
        const existing = acc[path];
        if (existing) {
            existing.declarations.push(declaration);
        }
        else {
            acc[path] = {
                declarations: [declaration],
            };
        }
        return acc;
    }, outputAcc);
};
exports.default = makeEnumsGenerator;
