"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const extract_pg_schema_1 = require("extract-pg-schema");
const rmfr_1 = __importDefault(require("rmfr"));
const default_metadata_generators_1 = require("./default-metadata-generators");
const defaultTypeMap_1 = __importDefault(require("./defaultTypeMap"));
const makeCompositeGenerator_1 = __importDefault(require("./generators/makeCompositeGenerator"));
const makeDomainsGenerator_1 = __importDefault(require("./generators/makeDomainsGenerator"));
const makeEnumsGenerator_1 = __importDefault(require("./generators/makeEnumsGenerator"));
const makeRangesGenerator_1 = __importDefault(require("./generators/makeRangesGenerator"));
const markAsGenerated_1 = __importDefault(require("./hooks/markAsGenerated"));
const render_1 = __importDefault(require("./render"));
const writeFile_1 = __importDefault(require("./writeFile"));
const processDatabase = (config, progress) => __awaiter(void 0, void 0, void 0, function* () {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
    const schemas = yield (0, extract_pg_schema_1.extractSchemas)(config.connection, Object.assign({ schemas: config.schemas, typeFilter: config.typeFilter }, progress));
    const typeMap = Object.assign(Object.assign({}, defaultTypeMap_1.default), config.customTypeMap);
    const getMetadata = (_a = config.getMetadata) !== null && _a !== void 0 ? _a : default_metadata_generators_1.defaultGetMetadata;
    const getPropertyMetadata = (_b = config.getPropertyMetadata) !== null && _b !== void 0 ? _b : default_metadata_generators_1.defaultGetPropertyMetadata;
    const generateIdentifierType = (_c = config.generateIdentifierType) !== null && _c !== void 0 ? _c : default_metadata_generators_1.defaultGenerateIdentifierType;
    const propertySortFunction = (_d = config.propertySortFunction) !== null && _d !== void 0 ? _d : default_metadata_generators_1.defaultPropertySortFunction;
    const instantiatedConfig = {
        getMetadata,
        getPropertyMetadata,
        generateIdentifierType,
        propertySortFunction,
        enumStyle: (_e = config.enumStyle) !== null && _e !== void 0 ? _e : 'enum',
        typeMap,
        schemas,
        connection: config.connection,
        outputPath: (_f = config.outputPath) !== null && _f !== void 0 ? _f : '.',
        preDeleteOutputFolder: (_g = config.preDeleteOutputFolder) !== null && _g !== void 0 ? _g : false,
        resolveViews: (_h = config.resolveViews) !== null && _h !== void 0 ? _h : true,
    };
    const generators = [
        (0, makeCompositeGenerator_1.default)('table', instantiatedConfig),
        (0, makeCompositeGenerator_1.default)('view', instantiatedConfig),
        (0, makeCompositeGenerator_1.default)('materializedView', instantiatedConfig),
        (0, makeCompositeGenerator_1.default)('compositeType', instantiatedConfig),
        (0, makeEnumsGenerator_1.default)(instantiatedConfig),
        (0, makeRangesGenerator_1.default)(instantiatedConfig),
        (0, makeDomainsGenerator_1.default)(instantiatedConfig),
    ];
    let output = {};
    Object.values(schemas).forEach((schema) => {
        generators.forEach((generator) => {
            output = generator(schema, output);
        });
    });
    const preRenderHooks = (_j = config.preRenderHooks) !== null && _j !== void 0 ? _j : [];
    for (const hook of preRenderHooks) {
        output = yield hook(output, instantiatedConfig);
    }
    let filesToWrite = Object.keys(output).map((path) => {
        const lines = (0, render_1.default)(output[path].declarations, path);
        return { fullPath: `${path}.ts`, lines };
    });
    const postRenderHooks = (_k = config.postRenderHooks) !== null && _k !== void 0 ? _k : [markAsGenerated_1.default];
    for (const hook of postRenderHooks) {
        filesToWrite = yield Promise.all(filesToWrite.map((file) => __awaiter(void 0, void 0, void 0, function* () {
            const lines = yield hook(file.fullPath, file.lines, instantiatedConfig);
            return Object.assign(Object.assign({}, file), { lines });
        })));
    }
    if (instantiatedConfig.preDeleteOutputFolder) {
        console.info(`Clearing old files in ${instantiatedConfig.outputPath}`);
        yield (0, rmfr_1.default)(instantiatedConfig.outputPath, { glob: true });
    }
    filesToWrite.forEach((file) => (0, writeFile_1.default)(file));
});
exports.default = processDatabase;
